# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_sofa_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
    convert_to_units_per_liter_explanation,
)


def compute_sofa_explanation(input_parameters):
    explanation = """
    The criteria for the SOFA Score are shown below:

       1. PaO₂/FiO₂ ratio (mm Hg): ≥400 = 0 points, 300-399 = +1 point,
        200-299 = +2 points, 100-199 (with respiratory support) = +3 points,
        <100 (with respiratory support) = +4 points
       2. Platelets (10^3/µL): ≥150 = 0 points, 100-149 = +1 point, 50-99 = +2
        points, 20-49 = +3 points, <20 = +4 points
       3. Glasgow Coma Scale (GCS): 15 = 0 points, 13-14 = +1 point, 10-12 =
        +2 points, 6-9 = +3 points, <6 = +4 points
       4. Bilirubin (mg/dL): <1.2 = 0 points, 1.2-1.9 = +1 point, 2.0-5.9 = +2
        points, 6.0-11.9 = +3 points, ≥12.0 = +4 points
       5. Mean arterial pressure (MAP) or administration of vasopressors:
        No hypotension = 0 points, MAP <70 mmHg = +1 point, Dopamine ≤5 or
        Dobutamine (any dose) = +2 points, Dopamine >5 or Epinephrine ≤0.1
        or norepinephrine ≤0.1 = +3 points, Dopamine >15 or Epinephrine >0.1
        or norepinephrine >0.1 = +4 points
       6. Creatinine (mg/dL) or urine output: <1.2 = 0 points, 1.2-1.9 = +1
       point, 2.0-3.4 = +2 points, 3.5-4.9 or urine output <500 mL/day = +3
       points, ≥5.0 or urine output <200 mL/day = +4 points

    The total SOFA Score is calculated by summing the points
    for each criterion.\n\n
    """

    explanation += "The patient's current SOFA score is 0.\n"

    sofa_score = 0

    pao2 = input_parameters["partial_pressure_oxygen"][0]
    fio2 = input_parameters["fio2"][0]

    dopamine = input_parameters.get("dopamine", [0])
    dobutamine = input_parameters.get("dobutamine", [0])
    epinephrine = input_parameters.get("epinephrine", [0])
    norepinephrine = input_parameters.get("norepinephrine", [0])

    explanation += (
        f"The patient's partial pressure of oxygen is {pao2} mm "
        f"Hg and FiO₂ percentage is {fio2} %. "
    )
    ratio = round_number(pao2 / fio2)
    explanation += (
        f"This means that the patient's partial pressure of "
        f"oxygen to FiO₂ ratio is {ratio}. "
    )

    if "mechanical_ventilation" not in input_parameters:
        explanation += (
            "Whether the patient is on mechanical ventillation "
            "is not reported and so we assume this to be false. "
        )
        input_parameters["mechanical_ventilation"] = False
    elif input_parameters["mechanical_ventilation"]:
        explanation += (
            "The patient is reported to be on mechanical " "ventillation. "
        )
    else:
        explanation += (
            "The patient is reported to not be on mechanical " "ventillation. "
        )
        input_parameters["mechanical_ventilation"] = False

    if "cpap" not in input_parameters:
        explanation += (
            "Whether the patient is on continuous positive "
            "airway pressure is not reported and so we assume "
            "this to be false. "
        )
        input_parameters["cpap"] = False
    elif input_parameters["cpap"]:
        explanation += (
            "The patient is reported to be using continuous "
            "positive airway pressure. "
        )
    else:
        explanation += (
            "The patient is reported to not be using continuous "
            "positive airway pressure. "
        )

    if 300 <= ratio < 400:
        explanation += (
            f"Because the patient's partial pressure of oxygen "
            f"to FiO₂ ratio is between 300 and 400, we increase "
            f"the score by one point, makeing the current total "
            f"{sofa_score} + 1 = {sofa_score + 1}.\n"
        )
        sofa_score += 1
    elif 200 <= ratio < 300:
        explanation += (
            f"Because the patient's partial pressure of oxygen "
            f"to FiO₂ ratio is between 200 and 300, we increase "
            f"the score by two points, makeing the current "
            f"total {sofa_score} + 2 = {sofa_score + 2}.\n"
        )
        sofa_score += 2
    elif ratio <= 199 and (
        not input_parameters["mechanical_ventilation"]
        and not input_parameters["cpap"]
    ):
        explanation += (
            f"Because the patient's partial pressure of oxygen "
            f"to FiO₂ ratio is between 200 and 300, the patient "
            f"is not on mechanical ventillation and is not using "
            f"continious positive airway pressure, we increase "
            f"the score by two points, makeing the current "
            f"total {sofa_score} + 2 = {sofa_score + 2}.\n"
        )
        sofa_score += 2
    elif 100 <= ratio < 199 and (
        input_parameters["mechanical_ventilation"] or input_parameters["cpap"]
    ):
        explanation += (
            f"Because the patient's partial pressure of oxygen "
            f"to FiO₂ ratio is between 100 to 199, and the "
            f"patient is using at least one of (i) mechanical "
            f"ventillation or (ii) continious positive airway "
            f"pressure, we increase the score by three points, "
            f"makeing the current total {sofa_score} + 3 = "
            f"{sofa_score + 3}.\n"
        )
        sofa_score += 3
    elif ratio < 100 and input_parameters["mechanical_ventilation"]:
        explanation += (
            f"Because the patient's partial pressure of oxygen "
            f"to FiO₂ ratio is less than 100, and the patient "
            f"is using at least one of (i) mechanical "
            f"ventillation or (ii) continious positive airway "
            f"pressure, we increase the score by four points, "
            f"makeing the current total {sofa_score} + 4 = "
            f"{sofa_score + 4}.\n"
        )
        sofa_score += 4

    if (
        'sys_bp' in input_parameters
        and 'dia_bp' in input_parameters
        and 1 / 3 * input_parameters['sys_bp'][0]
        + 2 / 3 * input_parameters['dia_bp'][0]
        < 70
        and (
            not dobutamine[0] and not epinephrine[0] and not norepinephrine[0]
        )
    ):
        sys_bp = input_parameters['sys_bp'][0]
        dia_bp = input_parameters['dia_bp'][0]
        map = round_number(1 / 3 * sys_bp + 2 / 3 * dia_bp)
        explanation = (
            f"The patient's systolic blood pressure is {sys_bp} "
            f"mm Hg and the patient's diastolic blood pressure "
            f"is {dia_bp} mm Hg, making the patient's mean "
            f"arterial blood pressure {map} mm Hg. "
        )
        explanation += (
            f"For one point to be given, the patient's mean "
            f"arterial pressure must be less than 70 mm Hg, "
            f"making the current total {sofa_score} + 1 "
            f"= {sofa_score + 1}.\n"
        )
        sofa_score += 1
    elif dopamine[0] <= 5 or dobutamine[0]:
        explanation += (
            f"For two points to be given, the patient must "
            f"be taking less than or equal to 5 micrograms/kg/min "
            f"or any amount of dobutamine. Because at least "
            f"one of these cases is true for the patient, "
            f"we increment the score by two points, "
            f"making the current total {sofa_score} + 2 "
            f"= {sofa_score + 2}.\n"
        )
        sofa_score += 2
    elif dopamine[0] > 5 or epinephrine[0] <= 0.1 or norepinephrine[0] <= 0.1:
        explanation += (
            f"For three points to be given, the patient "
            f"must be taking more than 5 micrograms/kg/min, "
            f"less than or equal to 0.1 micrograms/kg/min "
            f"of epinephrine, or less than or equal to 0.1 "
            f"micrograms/kg/min of norepinephrine. "
            f"Because at least one of these cases is true for the "
            f"patient, we increment the score by three "
            f"points, making the current total "
            f"{sofa_score} + 3 = {sofa_score + 3}.\n"
        )
        sofa_score += 3
    elif dopamine[0] > 15 or epinephrine[0] > 0.1 or norepinephrine[0] > 0.1:
        explanation += (
            f"For four points to be given, the patient "
            f"must be taking more than 15 micrograms/kg/min, "
            f"more than 0.1 micrograms/kg/min of epinephrine, "
            f"or more than 0.1 micrograms/kg/min of norepinephrine. "
            f"Because at least one of these cases is true "
            f"for the patient, we increment the score by "
            f"four points, making the current total "
            f"{sofa_score} + 4 = {sofa_score + 4}.\n"
        )
        sofa_score += 4

    if 'gcs' not in input_parameters:
        gcs = 15
        explanation += f"The patient's glasgow coma score is {gcs}. "
    else:
        gcs = input_parameters["gcs"]
        explanation += (
            "The patient's glasgow coma score is not "
            "reported so we take it to be 15. "
        )

    if gcs < 6:
        explanation += (
            f"Because the patient's glasgow coma score is "
            f"less than 6, we add 4 points to the score, "
            f"making the current score {sofa_score} + 4 "
            f"= {sofa_score + 4}.\n "
        )
        sofa_score += 4
    elif 6 <= gcs <= 9:
        explanation += (
            f"Because the patient's glasgow coma score "
            f"is between 6 and 9, we add 3 points to the "
            f"score, making the current score "
            f"{sofa_score} + 3 = {sofa_score + 3}.\n "
        )
        sofa_score += 3
    elif 10 <= gcs <= 12:
        explanation += (
            f"Because the patient's glasgow coma score "
            f"is between 10 and 12, we add 2 points to "
            f"the score, making the current score "
            f"{sofa_score} + 2 = {sofa_score + 2}.\n "
        )
        sofa_score += 2
    elif 13 <= gcs <= 14:
        explanation += (
            f"Because the patient's glasgow coma score "
            f"is between 13 and 14, we add 1 point to "
            f"the score, making the current score "
            f"{sofa_score} + 1 = {sofa_score + 1}.\n "
        )
        sofa_score += 1
    else:
        explanation += (
            f"Because the patient's glasgow coma score "
            f"is 15, we add 0 points to the score, "
            f"keeping the score at {sofa_score}.\n "
        )

    bilirubin_exp, bilirubin = conversion_explanation(
        input_parameters['bilirubin'][0],
        'bilirubin',
        584.66,
        None,
        input_parameters['bilirubin'][1],
        "mg/dL",
    )
    explanation += bilirubin_exp

    if bilirubin < 1.2:
        explanation += (
            f"Because the patient's bilirubin concentration is "
            f"less than 1.2 mg/dL, we add 0 points to the score, "
            f"keeping the score at {sofa_score}.\n "
        )
    if 1.2 <= bilirubin < 2.0:
        explanation += (
            f"Because the patient's bilirubin concentration is "
            f"at least 1.2 mg/dL but less than 2.0 mg/dL, "
            f"we increment the score by one point, "
            f"make the current score {sofa_score} + 1 = "
            f"{sofa_score + 1}.\n"
        )
        sofa_score += 1
    elif 2.0 <= bilirubin < 6.0:
        explanation += (
            f"Because the patient's bilirubin concentration is "
            f"at least 2.0 mg/dL but less than 6.0 mg/dL, "
            f"we increment the score by two points, "
            f"make the current score {sofa_score} + 2 "
            f"= {sofa_score + 2}.\n"
        )
        sofa_score += 2
    elif 6.0 <= bilirubin < 12.0:
        explanation += (
            f"Because the patient's bilirubin concentration "
            f"is at least 6.0 mg/dL but less than 12.0 mg/dL, "
            f"we increment the score by three points, "
            f"make the current score {sofa_score} + 3 "
            f"= {sofa_score + 3}.\n"
        )
        sofa_score += 3
    elif bilirubin >= 12.0:
        explanation += (
            f"Because the patient's bilirubin concentration is "
            f"at least 12.0 mg/dL, we increment the score by "
            f"four points, make the current score"
            f"{sofa_score} + 4 = {sofa_score + 4}.\n"
        )
        sofa_score += 4

    platelet_count_exp, platelet_count = (
        convert_to_units_per_liter_explanation(
            input_parameters["platelet_count"][0],
            input_parameters["platelet_count"][1],
            "platelet",
            "µL",
        )
    )
    explanation += platelet_count_exp

    if platelet_count >= 150000:
        explanation += (
            f"Because the patient's platelet count is at "
            f"least 150*10³/µL, we do not any points to "
            f"the score, keeping the current score "
            f"at {sofa_score}.\n"
        )
    if 100000 <= platelet_count < 150000:
        explanation += (
            f"Because the patient's platelet count is "
            f"between 100*10³/µL but less than 150*10³/µL, "
            f"we increment the score by one point, "
            f"making the current score {sofa_score} + 1 "
            f"= {sofa_score + 1}.\n"
        )
        sofa_score += 1
    elif 50000 <= platelet_count < 100000:
        explanation += (
            f"Because the patient's platelet count is "
            f"between 50*10³/µL but less than 100*10³/µL, "
            f"we increment the score by two points, "
            f"making the current score {sofa_score} + 2 "
            f"= {sofa_score + 2}.\n"
        )
        sofa_score += 2
    elif 20000 <= platelet_count < 50000:
        explanation += (
            f"Because the patient's platelet count is "
            f"between 20*10³/µL but less than 50*10³/µL, "
            f"we increment the score by three points, "
            f"making the current score {sofa_score} + 3 "
            f"= {sofa_score + 3}.\n"
        )
        sofa_score += 3
    elif platelet_count < 20000:
        explanation += (
            f"Because the patient's platelet count is "
            f"less than 20*10³/µL, we increment the score "
            f"by four points, making the current score "
            f"{sofa_score} + 4 = {sofa_score + 4}.\n"
        )
        sofa_score += 4

    if 'creatinine' not in input_parameters:
        urine_output = input_parameters["urine_output"][0]

        explanation += f"The patients urine output is {urine_output} mL/day. "

        if urine_output < 500:
            explanation += (
                f"Because the patient's urine output is "
                f"less than 500 mL/day, we increment the "
                f"score by three points, making the current "
                f"total {sofa_score} + 3 = {sofa_score + 3}.\n"
            )
            sofa_score += 3
        elif urine_output < 200:
            explanation += (
                f"Because the patient's urine output is "
                f"less than 200 mL/day, we increment the "
                f"score by four points, making the current "
                f"total {sofa_score} + 4 = {sofa_score + 4}.\n"
            )
            sofa_score += 4

    elif 'urine_output' not in input_parameters:
        creatinine_exp, creatinine = conversion_explanation(
            input_parameters['creatinine'][0],
            "creatinine",
            113.12,
            None,
            input_parameters['creatinine'][1],
            "mg/dL",
        )

        explanation += creatinine_exp

        if 1.2 <= creatinine < 2.0:
            explanation += (
                f"Because the patient's creatinine concentration "
                f"is at least 1.2 mg/dL but less than 2.0 mg/dL, "
                f"we increment the score by one point, "
                f"making the current total {sofa_score} + 1 "
                f"= {sofa_score + 1}.\n"
            )
            sofa_score += 1
        elif 2.0 <= creatinine < 3.5:
            explanation += (
                f"Because the patient's creatinine concentration "
                f"is at least 2.0 mg/dL but less than 3.5 mg/dL, "
                f"we increment the score by two points, making "
                f"the current total {sofa_score} + 2 = "
                f"{sofa_score + 2}.\n"
            )
            sofa_score += 2
        elif 3.5 <= creatinine < 5.0:
            explanation += (
                f"Because the patient's creatinine concentration "
                f"is at least 3.5 mg/dL but less than 5.0 mg/dL, "
                f"we increment the score by three points, "
                f"making the current total {sofa_score} + 3 "
                f"= {sofa_score + 3}.\n"
            )
            sofa_score += 3
        elif creatinine >= 5.0:
            explanation += (
                f"Because the patient's creatinine concentration "
                f"is greater than 5.0 mg/dL, "
                f"we increment the score by four points, "
                f"making the current total {sofa_score} + 4 "
                f"= {sofa_score + 4}.\n"
            )
            sofa_score += 4

    explanation += f"Hence, the patient's SOFA score is {sofa_score} points.\n"

    return {"Explanation": explanation, "Answer": sofa_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'partial_pressure_oxygen': (80, 'mm'),
            'fio2': (0.2, 'Hg'),
            'platelet_count': (150000, 'µL'),
            'gcs': 15,
            'dopamine': (97.2, "pg/mL"),
            'dobutamine': (15, "µg/kg/min"),
            'epinephrine': (19, "pg/mL"),
            'norepinephrine': (200, "pg/mL"),
            'cpap': False,
            'sys_bp': (70, "mm"),
            'dia_bp': (12, "Hg"),
            'bilirubin': (2.8, "mg/dL"),
            'creatinine': (3.7, 'mg/dL'),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_sofa_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
    'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function weight_conversion_explanation
- translation

Date: March 2025
"""
from . import ideal_body_weight
from .utils.rounding import round_number
from .utils.weight_conversion import (
    weight_conversion_explanation,
)


def abw_explanation(input_variables):
    r"""
    Calculates the patient's Adjusted Body Weight (ABW) and generates a
        detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
            key-value pairs:
            - "weight" (tuple): The patient's weight information in the format
                (value, unit).
                - Value (float): The numerical weight measurement.
                - Unit (str): The unit of weight, which can be "lbs" (pounds),
                    "g" (grams), or "kg" (kilograms).
            - "height" (tuple): The patient's height information in the format
                (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height, which can be "cm"
                    (centimeters) or "in" (inches).
            - "sex" (str): The patient's gender, which can be either "Male"
                or "Female".

    Returns:
        dict: Contains three key-value pairs:
            - "Explanation" (str): A detailed description of the calculation
                process, including IBW and ABW calculations.
            - "ABW" (str): The specific formula and result for ABW calculation.
            - "Answer" (float): The patient's adjusted body weight
                (in kilograms).

    Notes:
        - Uses the `weight_conversion.weight_conversion_explanation` function
            to convert weight to kilograms.
        - Uses the `ideal_body_weight.ibw_explanation` function to calculate
            Ideal Body Weight (IBW).
        - Uses the `round_number` function to round the result.

    Example:
        abw_explanation({'weight': (150, 'lbs'),
            'height': (170, 'cm'), 'sex': 'Male'})
        output: "{'Explanation': "The patient's gender
            is Male.\nThe patient's height is 170 cm,
            which is 170 cm * 0.393701 in/cm = 66.929
        in. \nFor males, the ideal body weight (IBW) is calculated as follows:
            \nIBW = 50 kg + 2.3 kg * (height (in inches) - 60)\nPlugging
        in the values gives us 50 kg + 2.3 kg * (66.929 (in inches) - 60)
            = 65.937 kg.\nHence, the patient's IBW is 65.937 kg.The
        patient's weight is 150 lbs so this converts to 150 lbs * 0.453592
            kg/lbs = 68.039 kg. To compute the ABW value, apply the
        following formula: ABW = IBW + 0.4 * (weight (in kg) - IBW (in kg)).
            ABW = 65.937 kg + 0.4 * (68.039 kg  -
        65.937 kg) = 66.778 kg. The patient's adjusted body weight is
            66.778 kg.\n", 'ABW': "To compute the ABW value,
            apply the following
        formula: ABW = IBW + 0.4 * (weight (in kg) - IBW (in kg)).
            ABW = 65.937 kg + 0.4 * (68.039 kg  - 65.937 kg) = 66.778 kg. The
        patient's adjusted body weight is 66.778 kg.\n", 'Answer': 66.778}"
    """

    weight_explanation, weight = weight_conversion_explanation(
        input_variables["weight"]
    )
    ibw_explanation = ideal_body_weight.ibw_explanation(input_variables)

    explanation = f"{ibw_explanation['Explanation']}"
    explanation += f"{weight_explanation}"

    ibw = ibw_explanation["Answer"]

    abw = round_number(ibw + 0.4 * (weight - ibw))
    abw_explanation_string = ""
    abw_explanation_string += (
        "To compute the ABW value, apply the following formula: "
    )
    abw_explanation_string += (
        "ABW = IBW + 0.4 * (weight (in kg) - IBW (in kg)). "
    )
    abw_explanation_string += (
        f"ABW = {ibw} kg + 0.4 * ({weight} kg  - {ibw} kg) = {abw} kg. "
    )
    abw_explanation_string += (
        f"The patient's adjusted body weight is {abw} kg.\n"
    )

    explanation += abw_explanation_string

    return {
        "Explanation": explanation,
        "ABW": abw_explanation_string,
        "Answer": abw,
    }


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "weight": (150, "lbs"),  # weight 150 lbs
            "height": (170, "cm"),  # height 170 cm
            "sex": "Male",  # Male
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = abw_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_cci_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)


def compute_cci_explanation(input_parameters):
    r"""
        Calculates the patient's Charlson Comorbidity Index (CCI) and generates
        a detailed explanatory text.

        Parameters:
            input_parameters (dict): A dictionary containing the following
            key-value pairs:
                - "age" (tuple): The patient's albumin concentration in the
                format (value, unit).
                    - Value (float): Age.
                    - Unit (str): The unit can be "months", "years".
                - "mi" (boolean): Myocardial infarction (history of definite or
                probable MI with EKG changes and/or enzyme changes)
                - "chf" (boolean): Congestive heart failure (CHF) (exertional
                or paroxysmal nocturnal dyspnea, responsive to digitalis,
                diuretics, or afterload reducing agents)
                - "peripheral_vascular_disease" (boolean): Peripheral vascular
                disease
                - "cva" (boolean): Cerebrovascular accident (CVA)
                - "tia" (boolean): transient ischemic attack (TIA)
                - "connective_tissue_disease" (boolean): Connective tissue
                    disease
                - "dementia" (boolean): Dementia
                - "copd" (boolean): Chronic obstructive pulmonary disease
                - "connective_tissue_disease" (boolean): Connective tissue
                    disease
                - "peptic_ucler_disease" (boolean): Peptic ulcer disease (any
                history of treatment for ulcer disease or ulcer bleeding)
                - "liver_disease" (str): Liver disease: None, Mild,
                Moderate to severe
                - "moderate_to_severe_ckd" (boolean): Moderate to severe
                chronic kidney disease
                - "diabetes_mellitus" (str): None, diet-controlled,
                    Uncomplicated,
                End-organ damage
                - "hemiplegia" (boolean): Hemiplegia
                - "solid_tumor" (str): None, Localized, Metastatic
                - "leukemia" (boolean): Leukemia
                - "lymphoma" (boolean): Lymphoma
                - "aids" (boolean): AIDS


        Returns:
            dict: Contains two key-value pairs:
                - "Explanation" (str): A detailed description of the
                calculation process.
                - "Answer" (float): The patient's Glomerular
                    Filtration Rate (GFR).

        Notes:
            - Uses the `conversion_explanation` function to convert source unit
            to target unit.

        Example:
            compute_cci_explanation({
                "age": (45, 'years'),
                "mi": False,
                "chf": False,
                "peripheral_vascular_disease": False,
                "cva": False,
                "tia": False,
                "connective_tissue_disease": False,
                "dementia": True,
                "copd": True,
                "hemiplegia": False,
                "peptic_ucler_disease": False,
                "diabetes_mellitus": "Uncomplicated",
                "moderate_to_severe_ckd": False,
                "solid_tumor": "Localized",
                "leukemia": False,
                "lymphoma": False,
                "aids": False,
                "liver_disease": "Mild",
            })

            output: "{'Explanation': "\n    The Charlson Comorbidity Index
            (CCI)
            are listed below:\n    \n       1. Age: <50 years = 0 points,
            50-59 years = +1 point, 60-69 years = +2 points, 70-79 years = +3
            points, ≥80 years = +4 points\n       2. Myocardial infarction (
            history of definite or probable MI with EKG changes and/or enzyme
            changes): No = 0 points, Yes = +1 point\n       3. Congestive heart
            failure (CHF) (exertional or paroxysmal nocturnal dyspnea,
            responsive to digitalis, diuretics, or afterload reducing agents):
            No = 0 points, Yes = +1 point\n       4. Peripheral vascular
            disease (intermittent claudication, past bypass for chronic
            arterial insufficiency, history of gangrene or acute arterial
            insufficiency, untreated thoracic or abdominal aneurysm ≥6 cm):
            No = 0 points, Yes = +1 point\n       5. Cerebrovascular accident
            (CVA) or transient ischemic attack (TIA) (history with minor or
            no residuals): No = 0 points, Yes = +1 point\n       6. Dementia
            (chronic cognitive deficit): No = 0 points, Yes = +1 point\n
            7. Chronic obstructive pulmonary disease (COPD): No = 0 points,
            Yes = +1 point\n       8. Connective tissue disease: No = 0 points,
            Yes = +1 point\n       9. Peptic ulcer disease (any history of
            treatment for ulcer disease or ulcer bleeding): No = 0 points,
            Yes = +1 point\n       10. Liver disease: None = 0 points, Mild =
            +1 point, Moderate to severe = +3 points\n       11. Diabetes
            mellitus: None or diet-controlled = 0 points, Uncomplicated = +1
            point, End-organ damage = +2 points\n       12. Hemiplegia: No =
            0 points, Yes = +2 points\n       13. Moderate to severe chronic
            kidney disease (CKD): No = 0 points, Yes = +2 points\n       14.
            Solid tumor: None = 0 points, Localized = +2 points, Metastatic =
            +6 points\n       15. Leukemia: No = 0 points, Yes = +2 points\n
            16. Lymphoma: No = 0 points, Yes = +2 points\n       17. AIDS: No =
            0 points, Yes = +6 points\n    \n    The total score is calculated
            by summing the points for each criterion.\\n\\n\n    The current
            CCI is value is 0.\nThe patient is 45 years old. Because the
            patient's age is less than 50, we do not add any points to the
            score,
            keeping the current total at 0.\nThe issue,
            'Myocardial infarction,'
            is reported to be absent for the patient and so we do not add any
            points to the score, keeping the current total at 0.\nThe patient's
            CCI score is 0 points.\nThe issue, 'Congestive heart failure,' is
            reported to be absent for the patient and so we do not add any
            points
            to the score, keeping the current total at 0.\nThe patient's CCI
            score is 0 points.\nThe issue, 'Peripheral vascular disease,' is
            reported to be absent for the patient and so we do not add any
            points to the score, keeping the current total at 0.\nThe patient's
            CCI score is 0 points.\nAt least one of transient ischemic attack
            or
            cerebral vascular accident must be present in the patient for a
            point
            to be added to the current total.\nTransient ischemic attacks is
            reported to be absent for the patient.\nCerebral vascular accident
            is
            reported to be absent for the patient.\nNeither of the issues are
            reported to be present for the patient and so we add 0 point to the
            score, keeping the current total at 0.\nThe issue, 'Connective
            tissue
            diease,' is reported to be absent for the patient and so we do not
            add any points to the score, keeping the current total at 0.\nThe
            patient's CCI score is 0 points.\n The issue,'Dementia,' is present
            for the patient and so we add 1 point to the score, making the
            current
            total 0 + 1 = 1.\nThe patient's CCI score is 1 points.\n The issue,
            'Chronic obstructive pulmonary disease,' is present for the patient
            and so we add 1 point to the score, making the current total
            1 + 1 =
            2.\nThe patient's CCI score is 2 points.\nThe issue, 'Hemiplegia,'
            is reported to be absent for the patient and so we do not add any
            points to the score, keeping the current total at 2.\nThe patient's
            CCI score is 2 points.\nThe issue, 'Peptic ulcer disease,' is
            reported
            to be absent for the patient and so we do not add any points to the
            score, keeping the current total at 2.\nThe patient's CCI score is
            2 points.\n The issue,'Diabetes mellitus,' is present for the
            patient
            and so we add 1 point to the score, making the current total
            2 + 1 = 3.
            \nThe patient's CCI score is 3 points.\nThe issue, 'Moderate to
            severe
            chronic kidney disease,' is reported to be absent for the patient
            and
            so we do not add any points to the score, keeping the current total
            at 3.\nThe patient's CCI score is 3 points.\n The issue,'Solid
            tumor,'
            is present for the patient and so we add 1 point to the score,
            making
            the current total 3 + 1 = 4.\nThe patient's CCI score is 4 points.
            \nThe
            issue, 'Leukemia,' is reported to be absent for the patient and
            so we
            do not add any points to the score, keeping
            the current total at 4.
            \nThe patient's CCI score is 4 points.\nThe issue, 'Lymphoma,' is
            reported to be absent for the patient and so we do not add any
            points
            to the score, keeping the current total at 4.\nThe patient's
            CCI score
            is 4 points.\nThe issue, 'AIDS,' is reported to be absent for the
            patient and so we do not add any points to the score, keeping the
            current total at 4.\nThe patient's CCI score is 4 points.\n The
            issue,'Liver Disease,' is present for the patient and so we add 1
            point to the score, making the current total 4 + 1 = 5.\nThe
            patient's CCI score is 5 points.\n", 'Answer': 5}
    "
    """

    parameter_to_name = {
        "mi": "Myocardial infarction",
        'chf': "Congestive heart failure",
        "peripheral_vascular_disease": "Peripheral vascular disease",
        "cva": "Cerebrovascular accident",
        "tia": "Transient ischemic attacks",
        'connective_tissue_disease': "Connective tissue diease",
        "dementia": "Dementia",
        "copd": "Chronic obstructive pulmonary disease",
        'hemiplegia': "Hemiplegia",
        "peptic_ucler_disease": "Peptic ulcer disease",
        "diabetes_mellitus": "Diabetes mellitus",
        "moderate_to_severe_ckd": 'Moderate to severe chronic kidney disease',
        "solid_tumor": "Solid tumor",
        "leukemia": "Leukemia",
        "lymphoma": "Lymphoma",
        "aids": "AIDS",
        "liver_disease": "Liver Disease",
    }

    two_point_params = [
        'hemiplegia',
        "moderate_to_severe_ckd",
        "leukemia",
        "lymphoma",
    ]

    explanation = r"""
    The Charlson Comorbidity Index (CCI) are listed below:
       1. Age: <50 years = 0 points, 50-59 years = +1 point, 60-69 years = +2
        points, 70-79 years = +3 points, ≥80 years = +4 points
       2. Myocardial infarction (history of definite or probable MI with EKG
        changes and/or enzyme changes): No = 0 points, Yes = +1 point
       3. Congestive heart failure (CHF) (exertional or paroxysmal nocturnal
        dyspnea, responsive to digitalis, diuretics, or afterload reducing
        agents): No = 0 points, Yes = +1 point
       4. Peripheral vascular disease (intermittent claudication, past bypass
        for chronic arterial insufficiency, history of gangrene or acute
        arterial insufficiency, untreated thoracic or abdominal
        aneurysm ≥6 cm): No = 0 points, Yes = +1 point
       5. Cerebrovascular accident (CVA) or transient ischemic attack (TIA)
        (history with minor or no residuals): No = 0 points, Yes = +1 point
       6. Dementia (chronic cognitive deficit): No = 0 points, Yes = +1 point
       7. Chronic obstructive pulmonary disease (COPD): No = 0 points,
        Yes = +1 point
       8. Connective tissue disease: No = 0 points, Yes = +1 point
       9. Peptic ulcer disease (any history of treatment for ulcer
        disease or ulcer bleeding): No = 0 points, Yes = +1 point
       10. Liver disease: None = 0 points, Mild = +1 point,
        Moderate to severe = +3 points
       11. Diabetes mellitus: None or diet-controlled = 0 points,
        Uncomplicated = +1 point, End-organ damage = +2 points
       12. Hemiplegia: No = 0 points, Yes = +2 points
       13. Moderate to severe chronic kidney disease (CKD):
        No = 0 points, Yes = +2 points
       14. Solid tumor: None = 0 points, Localized = +2 points,
        Metastatic = +6 points
       15. Leukemia: No = 0 points, Yes = +2 points
       16. Lymphoma: No = 0 points, Yes = +2 points
       17. AIDS: No = 0 points, Yes = +6 points

    The total score is calculated by summing the points for each criterion.\n\n
    """

    age_exp, age = age_conversion_explanation(input_parameters["age"])
    explanation += "The current CCI is value is 0.\n"
    explanation += age_exp
    cci = 0

    if age < 50:
        explanation += (
            f"Because the patient's age is less than 50, "
            f"we do not add any points to the score, "
            f"keeping the current total at {cci}.\n"
        )
    elif 49 < age < 60:
        explanation += (
            f"Because the patient's age is between 50 and 59, "
            f"we add 1 point to the score, "
            f"making the current total = {cci} + 1 = {cci + 1}.\n"
        )
        cci += 1
    elif 59 < age < 70:
        explanation += (
            f"Because the patient's age is between 60 and 69, "
            f"we add 2 points to the score, "
            f"making the current total = {cci} + 2 = {cci + 2}.\n"
        )
        cci += 2
    elif 69 < age < 80:
        explanation += (
            f"Because the patient's age is between 70 and 79, "
            f"we add 3 points to the score, "
            f"making the current total = {cci} + 3 = {cci + 3}.\n"
        )
        cci += 3
    elif age >= 80:
        explanation += (
            f"Because the patient's age is greater than "
            f"or equal to 80 years, we add 4 points to "
            f"the score, making the current "
            f"total = {cci} + 4 = {cci + 4}.\n"
        )
        cci += 4

    for parameter in parameter_to_name:
        if parameter == "tia":
            continue

        if parameter == "cva":
            explanation += (
                "At least one of transient ischemic attack "
                "or cerebral vascular accident must be present "
                "in the patient for a point to be "
                "added to the current total.\n"
            )

            if 'tia' not in input_parameters:
                explanation += (
                    "Transient ischemic attacks is not reported "
                    "for the patient and so we assume it to be "
                    "absent.\n"
                )
                input_parameters["tia"] = False
            elif input_parameters['tia']:
                explanation += (
                    "Transient ischemic attacks is reported to "
                    "be present for the patient.\n"
                )
            else:
                explanation += (
                    "Transient ischemic attacks is reported to "
                    "be absent for the patient.\n"
                )

            if 'cva' not in input_parameters:
                explanation += (
                    "Cerebral vascular accident is not reported "
                    "for the patient and so we assume it to be "
                    "absent.\n"
                )
                input_parameters["cva"] = False
            elif input_parameters['cva']:
                explanation += (
                    "Cerebral vascular accident is reported to "
                    "be present for the patient.\n"
                )
            else:
                explanation += (
                    "Cerebral vascular accident is reported to "
                    "be absent for the patient.\n"
                )

            if input_parameters['cva'] or input_parameters['tia']:
                explanation += (
                    f"Because at least one of the issues is "
                    f"reported to be present for the patient, "
                    f"we add 1 point to the score, making the "
                    f"current total {cci} + 1 = {cci + 1}.\n"
                )
                cci += 1
                continue
            else:
                explanation += (
                    f"Neither of the issues are reported to be "
                    f"present for the patient and so we add 0 "
                    f"point to the score, keeping the current "
                    f"total at {cci}.\n"
                )
                continue

        if parameter == 'solid_tumor' and parameter not in input_parameters:
            explanation += (
                f"The patient's solid tumor status is not "
                f"reported and so we assume that it is 'none.' "
                f"Hence, do not add any points to the score, "
                f"keeping the current total at {cci}.\n"
            )
            continue
        elif (
            parameter == 'solid_tumor'
            and parameter in input_parameters
            and input_parameters[parameter] == 'none'
        ):
            explanation += (
                f"The patient's solid tumor is reported to be "
                f"'none' and so we do not add any points to the "
                f"score, keeping the current total at {cci}.\n"
            )
            continue
        elif (
            parameter == 'solid_tumor'
            and parameter in input_parameters
            and input_parameters[parameter] == 'localized'
        ):
            explanation += (
                f"The patient's solid tumor is reported to be "
                f"'localized' and so we add 2 points to the "
                f"score, making the current total {cci} + 2 = "
                f"{cci + 2}.\n"
            )
            cci += 2
            continue
        elif (
            parameter == 'solid_tumor'
            and parameter in input_parameters
            and input_parameters[parameter] == 'metastatic'
        ):
            explanation += (
                f"The patient's solid tumor is reported to be "
                f"'metastatic' and so we add 6 points to the "
                f"score, making the current total {cci} + 6 = "
                f"{cci + 6}.\n"
            )
            cci += 6
            continue

        if parameter == 'liver_diease' and parameter not in input_parameters:
            explanation += (
                f"The patient's liver disease status is not "
                f"reported and so we assume the value to be "
                f"'none or diet-controlled.' No points are added "
                f"to the score, keeping the current total at "
                f"{cci}.\n"
            )
            continue
        elif (
            parameter == 'liver_disease'
            and parameter in input_parameters
            and input_parameters[parameter] == 'none'
        ):
            explanation += (
                f"The patient's liver disease is reported to be "
                f"'none' and so we do not add any points to the "
                f"score, keeping the current total at {cci}.\n"
            )
            continue
        elif (
            parameter == 'liver_disease'
            and parameter in input_parameters
            and input_parameters[parameter] == 'mild'
        ):
            explanation += (
                f"The patient's liver disease is reported to be "
                f"'mild' and so we add 1 point to the score, "
                f"making the current total {cci} + 1 = {cci + 1}.\n"
            )
            cci += 1
            continue
        elif (
            parameter == 'liver_disease'
            and parameter in input_parameters
            and input_parameters[parameter] == 'moderate to severe'
        ):
            explanation += (
                f"The patient's liver disease is reported to be "
                f"'moderate to severe' and so we add 3 points to "
                f"the score, making the current "
                f"total {cci} + 3 = {cci + 3}.\n"
            )
            cci += 3
            continue

        if (
            parameter == 'diabetes_mellitus'
            and 'diabetes_mellitus' not in input_parameters
        ):
            explanation += (
                f"The patient's diabetes mellitus status is not "
                f"reported and so we assume the value to be "
                f"'none or diet-controlled.' No points are added "
                f"to the score, keeping the current total at "
                f"{cci}.\n"
            )
            continue
        elif (
            parameter == 'diabetes_mellitus'
            and parameter in input_parameters
            and input_parameters[parameter] == 'none or diet-controlled'
        ):
            explanation += (
                f"The patient's diabetes mellitus is reported to "
                f"be 'none or diet-controlled' and so we add 0 "
                f"point to the score, keeping the current total "
                f"at {cci}.\n"
            )
            continue
        elif (
            parameter == 'diabetes_mellitus'
            and parameter in input_parameters
            and input_parameters[parameter] == 'uncomplicated'
        ):
            explanation += (
                f"The patient's diabetes mellitus is reported to "
                f"be 'uncomplicated' and so we add 1 point "
                f"to the score, making the current "
                f"total {cci} + 1 = {cci + 1}.\n"
            )
            cci += 1
            continue
        elif (
            parameter == 'diabetes_mellitus'
            and parameter in input_parameters
            and input_parameters[parameter] == 'end-organ damage'
        ):
            explanation += (
                f"The patient's diabetes mellitus is reported to "
                f"be 'end-organ damage' and so we add 2 points "
                f"to the score, making the current total {cci} + "
                f"2 = {cci + 2}.\n"
            )
            cci += 2
            continue

        if (
            parameter in two_point_params
            and parameter in input_parameters
            and input_parameters[parameter]
        ):
            explanation += (
                f"The issue, '{parameter_to_name[parameter]},"
                f"' is reported to be present for the patient "
                f"and so we add 2 points to the score, "
                f"making the current total {cci} + 2 = {cci + 2}.\n"
            )
            cci += 2
        elif (
            parameter == 'aids'
            and parameter in input_parameters
            and input_parameters['aids']
        ):
            explanation += (
                f'AIDS is reported to be present for the patient '
                f'and so we add 6 points to the score, '
                f'making the current total at '
                f'{cci} + 6 = {cci + 6}.\n'
            )
            cci += 6

        elif (
            parameter not in two_point_params
            and parameter != 'aids'
            and parameter in input_parameters
            and input_parameters[parameter]
        ):
            explanation += (
                f" The issue,'{parameter_to_name[parameter]},"
                f"' is present for the patient and so we add 1 "
                f"point to the score, making the current total "
                f"{cci} + 1 = {cci + 1}.\n"
            )
            cci += 1
        elif parameter in input_parameters and not input_parameters[parameter]:
            explanation += (
                f"The issue, '{parameter_to_name[parameter]},"
                f"' is reported to be absent for the patient and "
                f"so we do not add any points to the score, "
                f"keeping the current total at {cci}.\n"
            )
        elif parameter not in input_parameters:
            explanation += (
                f"The issue, '{parameter_to_name[parameter]},"
                f"' is reported to be absent for the patient and "
                f"so we do not add any points to the score, "
                f"keeping the current total at {cci}.\n"
            )

        explanation += f"The patient's CCI score is {cci} points.\n"

    return {"Explanation": explanation, "Answer": cci}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "age": (45, 'years'),
            "mi": False,
            "chf": False,
            "peripheral_vascular_disease": False,
            "cva": False,
            "tia": False,
            "connective_tissue_disease": False,
            "dementia": True,
            "copd": True,
            "hemiplegia": False,
            "peptic_ucler_disease": False,
            "diabetes_mellitus": "Uncomplicated",
            "moderate_to_severe_ckd": False,
            "solid_tumor": "Localized",
            "leukemia": False,
            "lymphoma": False,
            "aids": False,
            "liver_disease": "Mild",
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_cci_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_fever_pain_explanation
- translation

Date: March 2025
"""


def compute_fever_pain_explanation(input_parameters):
    r"""
    Calculates the patient's FeverPAIN score and generates a detailed
    explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "symptom_onset" (Optional[bool]): Whether the patient has a
            fever in the past 24 hours.
            - "purulent_tonsils" (Optional[bool]): Presence of pus on the
            tonsils.
            - "fever_24_hours" (Optional[bool]): Whether the patient has
            had a fever in the past 24 hours.
            - "severe_tonsil_inflammation" (Optional[bool]): Presence of
            severe tonsil inflammation.
            - "cough_coryza_absent" (Optional[bool]): Absence of cough or
            coryza.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's FeverPAIN score.

    Notes:
        - None

    Example:
        compute_fever_pain_explanation({'symptom_onset': True,
        'fever_24_hours': True, 'cough_coryza_absent': True})

        output: "{'Explanation': "\n    The criteria for the FeverPAIN score
        are listed below:\n\n 1. Fever in past 24 hours: No = 0 points,
        Yes = +1 point\n 2. Absence of cough or coryza: No = 0 points,
        Yes = +1 point\n 3. Symptom onset ≤3 days: No = 0 points, Yes = +1
        point\n 4. Purulent tonsils: No = 0 points, Yes = +1 point\n 5.
        Severe tonsil inflammation: No = 0 points, Yes = +1 point\n\n The
        FeverPAIN score is calculated by summing the points for each
        criterion.\n\n\n The patient's current FeverPain score is 0.\n'The
        patient is reported to have a fever in the past 24 hours and so we
        increment the score by 1, making the current total 0 + 1 = 1.
        \n'The patient is reported to have an absence of cough or coryza
        and so we increment the score by 1, making the current total
        1 + 1 = 2.\n'The patient is reported to have a symptom onset ≤3 days
        and so we increment the score by 1, making the current total
        2 + 1 = 3. \nWhether the patient has purulent tonsils is not
        reported and so we assume that it is absent for the patient.
        Because of this, we do not increment the score, keeping the current
        total at 3. \nWhether the patient has severe tonsil inflammation is
        not reported and so we assume that it is absent for the patient.
        Because of this, we do not increment the score, keeping the current
        total at 3.\nThe patient's FeverPain score is 3 points.\n",
        'Answer': 3}"
    """

    parameter_name = {
        "fever_24_hours": "a fever in the past 24 hours",
        "cough_coryza_absent": "an absence of cough or coryza",
        "symptom_onset": "a symptom onset ≤3 days",
        "purulent_tonsils": "purulent tonsils",
        "severe_tonsil_inflammation": "severe tonsil inflammation",
    }

    fever_pain_score = 0

    explanation = """
     The criteria for the FeverPAIN score are listed below:

        1. Fever in past 24 hours: No = 0 points, Yes = +1 point
        2. Absence of cough or coryza: No = 0 points, Yes = +1 point
        3. Symptom onset ≤3 days: No = 0 points, Yes = +1 point
        4. Purulent tonsils: No = 0 points, Yes = +1 point
        5. Severe tonsil inflammation: No = 0 points, Yes = +1 point

     The FeverPAIN score is calculated by summing the points for each 
     criterion.\n\n
     """

    explanation += "The patient's current FeverPain score is 0.\n"

    for parameter in parameter_name:
        if parameter not in input_parameters:
            explanation += (
                f"Whether the patient has {parameter_name[parameter]} is "
                f"not reported and so we assume that it is absent for the "
                f"patient. Because of this, we do not increment the score, "
                f"keeping the current total at {fever_pain_score}.\n"
            )

        elif input_parameters[parameter]:
            explanation += (
                f"'The patient is reported to have "
                f"{parameter_name[parameter]} and so we increment the score "
                f"by 1, making the current total {fever_pain_score} + 1 = "
                f"{fever_pain_score + 1}.\n"
            )
            fever_pain_score += 1

        else:
            explanation += (
                f"The patient is reported to not have "
                f"{parameter_name[parameter]} and so we do not "
                f"increment the score, keeping the current total at "
                f"{fever_pain_score}.\n"
            )

    explanation += (
        f"The patient's FeverPain score is {fever_pain_score} points.\n"
    )

    return {"Explanation": explanation, "Answer": fever_pain_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "symptom_onset": True,
            "purulent_tonsils": False,
            "fever_24_hours": True,
            "severe_tonsil_inflammation": False,
            "cough_coryza_absent": False,
        },
        {
            'symptom_onset': True,
            'fever_24_hours': True,
            'cough_coryza_absent': True,
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_fever_pain_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_wells_criteria_dvt_explanation
- translation

Date: March 2025
"""


def compute_wells_criteria_dvt_explanation(input_parameters):
    r"""
    Calculates the patient's score of Wells' criteria for DVT and generates
    a detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "active_cancer" (boolean): Whether the patient has active
            cancer or not.
            - "bedridden_for_atleast_3_days" (boolean): bedridden recently
            >3 days.
            - "major_surgery_in_last_12_weeks" (boolean): major surgery
            within 12 weeks.
            - "calf_swelling_3cm" (boolean): calf swelling >3 cm compared
            to the other leg.
            - "collateral_superficial_veins" (boolean): collateral (
            nonvaricose) superficial veins present.
            - "leg_swollen" (boolean): entire leg swollen.
            - "localized_tenderness_on_deep_venuous_system" (boolean):
            localized tenderness along the deep venous
            system.
            - "pitting_edema_on_symptomatic_leg" (boolean): pitting edema,
            confined to symptomatic leg.
            - "paralysis_paresis_immobilization_in_lower_extreme" (boolean):
            paralysis, paresis, or recent plaster
            immobilization of the lower extremity
            - "previous_dvt" (boolean): Previous, objectively diagnosed DVT:
            No = 0 points, Yes = +1.5 points.
            - "alternative_to_dvt_diagnosis" (boolean): alternative
            diagnosis to DVT as likely or more likely.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's score of Wells' criteria for
            Pulmonary Embolism.

    Notes:
        - None.

    Example:
        calculate_pe_wells_explanation({
            "active_cancer": False,
            "bedridden_for_atleast_3_days": False,
            "major_surgery_in_last_12_weeks": False,
            "calf_swelling_3cm": False,
            "collateral_superficial_veins": True,
            "leg_swollen": False,
            "localized_tenderness_on_deep_venuous_system": False,
            "pitting_edema_on_symptomatic_leg": False,
            "paralysis_paresis_immobilization_in_lower_extreme": False,
            "previous_dvt": True,
            "alternative_to_dvt_diagnosis": False,
        })

        output: "{'Explanation': "\n    The criteria for the Wells' Criteria
        for Deep Vein Thrombosis (DVT) score are listed below:\n\n
        1. Active cancer (treatment or palliation within 6 months):
        No = 0 points, Yes = +1 point\n       2. Bedridden recently >3 days
        or major surgery within 12 weeks: No = 0 points, Yes = +1 point
        \n       3. Calf swelling >3 cm compared to the other leg (measured
        10 cm below tibial tuberosity): No = 0 points, Yes = +1 point
        \n       4. Collateral (nonvaricose) superficial veins present:
        No = 0 points, Yes = +1 point\n       5. Entire leg swollen:
        No = 0 points, Yes = +1 point\n 6. Localized tenderness along the
        deep venous system: No = 0 points, Yes = +1 point\n 7. Pitting
        edema, confined to symptomatic leg: No = 0 points, Yes = +1 point
        \n 8. Paralysis, paresis, or recent plaster immobilization of the
        lower extremity: No = 0 points, Yes = +1 point\n
        9. Previously documented DVT: No = 0 points, Yes = +1 point\n 10.
        Alternative diagnosis to DVT as likely or more likely: No = 0
        points, Yes = -2 points\n \n    The total score is calculated by
        summing the points for each criterion.\\n\\n\n The current Well's
        DVT Score is 0.\nFrom the patient's note, the issue, 'active cancer,
        ' is absent. By the Well's DVT rule, a point should not be given,
        and so the score remains unchanged and so total remains at 0.\nFrom
        the patient's note, the issue, 'bedridden recently >3 days,
        ' is absent. From the patient's note, the issue, 'major surgery
        within 12 weeks,' is absent. Based on the Well's DVT rule, at least
        one of the issues, 'bedridden recently >3 days' or 'major surgery
        within 12 weeks' must be true for this criteria to be met for the
        score to increase by 1. This is not the case for this patient,
        and so the score remains unchanged at 0.\nFrom the patient's note,
        the issue, 'calf swelling >3 cm compared to the other leg,'
        is absent. By the Well's DVT rule, a point should not be given,
        and so the score remains unchanged and so total remains at 0.\nFrom
        the patient's note, the issue, 'collateral (nonvaricose)
        superficial veins present,' is present. By Well's DVT rule, a point
        should be given, and so we increment the score by one, making
        the total 0 + 1 =  1.\nFrom the patient's note, the issue, 'entire
        leg swollen,' is absent. By the Well's DVT rule, a point should not
        be given, and so the score remains unchanged and so total remains
        at 1.\nFrom the patient's note, the issue, 'localized tenderness
        along the deep venous system,' is absent. By the Well's DVT rule,
        a point should not be given, and so the score remains unchanged and
        so total remains at 1. \nFrom the patient's note, the issue,
        'pitting edema, confined to symptomatic leg,' is absent. By the
        Well's DVT rule, a point should not be given, and so the score
        remains unchanged and so total remains at 1.\nFrom the patient's
        note, the issue, 'paralysis, paresis, or recent plaster
        immobilization of the lower extremity,' is absent. By the Well's
        DVT rule, a point should not be given, and so the score remains
        unchanged and so total remains at 1.\nFrom the patient's note,
        the issue, 'previously documented DVT,' is present. By Well's DVT
        rule, a point should be given, and so we increment the score by
        one, making the total 1 + 1 =  2.\nFrom the patient's note,
        the issue, 'alternative diagnosis to DVT as likely or more likely,
        ' is absent. By the Well's DVT rule, a point should not be given,
        and so the score remains unchanged and so total remains at 2.\nThe
        Well's DVT score for the patient is 2.\n", 'Answer': 2}"
    """

    # List of parameters and their default values
    parameters = [
        ('active_cancer', "active cancer"),
        ('bedridden_for_atleast_3_days', "bedridden recently >3 days"),
        ('major_surgery_in_last_12_weeks', "major surgery within 12 weeks"),
        ('calf_swelling_3cm', "calf swelling >3 cm compared to the other leg"),
        (
            'collateral_superficial_veins',
            "collateral (nonvaricose) superficial veins present",
        ),
        ('leg_swollen', "entire leg swollen"),
        (
            'localized_tenderness_on_deep_venuous_system',
            "localized tenderness along the deep venous system",
        ),
        (
            'pitting_edema_on_symptomatic_leg',
            "pitting edema, confined to symptomatic leg",
        ),
        (
            'paralysis_paresis_immobilization_in_lower_extreme',
            "paralysis, paresis, or "
            "recent plaster immobilization of the lower extremity",
        ),
        ('previous_dvt', 'previously documented DVT'),
        (
            'alternative_to_dvt_diagnosis',
            "alternative diagnosis to DVT as likely or more likely",
        ),
    ]

    output = r"""
    The criteria for the Wells' Criteria for Deep Vein Thrombosis (DVT)
        score are listed below:

       1. Active cancer (treatment or palliation within 6 months): No = 0
        points, Yes = +1 point
       2. Bedridden recently >3 days or major surgery within 12 weeks: No = 0
        points, Yes = +1 point
       3. Calf swelling >3 cm compared to the other leg (measured 10 cm below
        tibial tuberosity): No = 0 points, Yes = +1 point
       4. Collateral (nonvaricose) superficial veins present: No = 0 points,
        Yes = +1 point
       5. Entire leg swollen: No = 0 points, Yes = +1 point
       6. Localized tenderness along the deep venous system: No = 0 points,
        Yes = +1 point
       7. Pitting edema, confined to symptomatic leg: No = 0 points,
        Yes = +1 point
       8. Paralysis, paresis, or recent plaster immobilization of
        the lower extremity: No = 0 points, Yes = +1 point
       9. Previously documented DVT: No = 0 points, Yes = +1 point
       10. Alternative diagnosis to DVT as likely or more likely:
        No = 0 points, Yes = -2 points

    The total score is calculated by summing the points for each criterion.\n\n
    """

    # Initializing points and output explanation
    score = 0
    output += "The current Well's DVT Score is 0.\n"

    count = 0

    while count < len(parameters):
        param_name = parameters[count][0]

        param_value = input_parameters.get(param_name)

        # If parameter is missing, assume it as False
        if param_value is None:
            output += (
                f"The issue,'{parameters[count][1]},' is missing from the "
                f"patient note and so the value is assumed "
                f"to be absent from the patient. "
            )
            input_parameters[param_name] = False
            param_value = False

        else:
            param_value_name = 'absent' if not param_value else 'present'
            output += (
                f"From the patient's note, the issue, '"
                f"{parameters[count][1]},' is {param_value_name}. "
            )

        if param_name == 'bedridden_for_atleast_3_days':
            count += 1
            continue
        if param_name == 'major_surgery_in_last_12_weeks':
            if (
                input_parameters['bedridden_for_atleast_3_days']
                or input_parameters['major_surgery_in_last_12_weeks']
            ):
                output += (
                    f"Based on the Well's DVT rule, at least one of the "
                    f"issues, 'bedridden recently >3 days' or "
                    f"'major surgery within 12 weeks' must be true for this "
                    f"criteria to be met for the score to increase by 1. "
                    f"Because this is the case, we incease the score by "
                    f"one making the total {score} + 1 = {score + 1}.\n"
                )
                score += 1
            else:
                output += (
                    f"Based on the Well's DVT rule, at least one of the "
                    f"issues, 'bedridden recently >3 days' or "
                    f"'major surgery within 12 weeks' must be true for this "
                    f"criteria to be met for the score "
                    f"to increase by 1. This is not the case for this "
                    f"patient, and so the score remains unchanged at "
                    f"{score}.\n"
                )
            count += 1
            continue

        # Score calculation
        if param_value:
            if param_name == 'alternative_to_dvt_diagnosis':
                output += (
                    f"By the Well's DVT rule, we decrease the score "
                    f"by 2 and so total is {score} - 2 = {score - 2}.\n"
                )
                score -= 2
                count += 1
                continue
            else:
                output += (
                    f"By Well's DVT rule, a point should be given, "
                    f"and so we increment the score by one, "
                    f"making the total {score} + 1 =  {score + 1}.\n"
                )
                score += 1
        elif param_value is False:
            output += (
                f"By the Well's DVT rule, a point should not be given, "
                f"and so the score remains unchanged "
                f"and so total remains at {score}.\n"
            )

        count += 1

    output += f"The Well's DVT score for the patient is {score}.\n"

    return {"Explanation": output, "Answer": score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "active_cancer": False,
            "bedridden_for_atleast_3_days": False,
            "major_surgery_in_last_12_weeks": False,
            "calf_swelling_3cm": False,
            "collateral_superficial_veins": True,
            "leg_swollen": False,
            "localized_tenderness_on_deep_venuous_system": False,
            "pitting_edema_on_symptomatic_leg": False,
            "paralysis_paresis_immobilization_in_lower_extreme": False,
            "previous_dvt": True,
            "alternative_to_dvt_diagnosis": False,
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_wells_criteria_dvt_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function targetweight_explanation
- translation

Date: March 2025
"""

import math

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def mrdr_gfr_explanation(input_variables):
    r"""
    Calculates the patient's Glomerular Filtration Rate (GFR) and generates
    a detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "age" (tuple): The patient's blood creatinine level in the
            format (value, unit).
                - Value (float): blood creatinine level.
                - Unit (str): The unit can be "mg/dL", "µmol/L", and so on.
            - "creatinine" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height, which can be "cm" (
                centimeters) or "in" (inches).
            - "sex" (str): The patient's gender, which can be either "Male"
            or "Female".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Glomerular Filtration Rate (GFR).

    Notes:
        - Uses the `conversion_explanation` function to convert source unit
        to target unit.

    Example:
        mrdr_gfr_explanation({"age": (49, 'years'),
        "creatinine": (10.6, 'mg/dL'),"sex": "Male"})

        output: "{'Explanation': "The patient is 49 years old. The
        concentration of Creatinine is 10.6 mg/dL. \nThe race of
        the patient is not provided, so the default value of the race
        coefficient is 1.0. \nThe patient is male, so the
        gender coefficient is 1.\nThe patient's estimated GFR is calculated
        using the MDRD equation as: \nGFR = 175 * creatinine^(-1.154) *
        age^(-0.203) * race_coefficient * gender_coefficient.
        The creatinine concentration is mg/dL.\nPlugging in these values
        will give us: 175 * 10.6^(-1.154) * 49^(-0.203) * 1 * 1=5.209.
        \nHence, the patient's GFR is 5.209 mL/min/1.73m².\n", 'Answer':
        5.209}"
    """

    gender = input_variables["sex"]

    age_explanation, age = age_conversion_explanation(input_variables["age"])
    creatinine_exp, creatinine_conc = conversion_explanation(
        input_variables["creatinine"][0],
        "Creatinine",
        113.12,
        None,
        input_variables["creatinine"][1],
        "mg/dL",
    )

    explanation = ""
    explanation += f"{age_explanation}"
    explanation += f"{creatinine_exp}\n"

    race_coefficient = 1

    if "race" in input_variables:
        race = input_variables["race"]
        if race == "Black":
            race_coefficient = 1.212
            explanation += (
                "The patient is Black, so the race coefficient is 1.212.\n"
            )
        else:
            explanation += (
                "The patient is not Black, so the race "
                "coefficient is defaulted to 1.0.\n"
            )
    else:
        explanation += (
            "The race of the patient is not provided, so the "
            "default value of the race coefficient is 1.0.\n"
        )

    gender_coefficient = 1
    if gender == "Female":
        gender_coefficient = 0.742
        explanation += (
            "The patient is female, so the gender coefficient is 0.742.\n"
        )
    else:
        explanation += "The patient is male, so the gender coefficient is 1.\n"

    gfr = round_number(
        175
        * math.exp(math.log(creatinine_conc) * -1.154)
        * math.exp(math.log(age) * -0.203)
        * race_coefficient
        * gender_coefficient
    )

    explanation += (
        f"The patient's estimated GFR is calculated using the MDRD equation "
        f"as:\nGFR = 175 * creatinine^(-1.154) * age^(-0.203) * "
        f"race_coefficient * gender_coefficient. "
        f"The creatinine concentration is mg/dL.\n"
        f"Plugging in these values will give us: "
        f"175 * {creatinine_conc}^(-1.154) * {age}^(-0.203) * "
        f"{race_coefficient} * {gender_coefficient}={gfr}.\n"
        f"Hence, the patient's GFR is {gfr} mL/min/1.73m².\n"
    )

    return {"Explanation": explanation, "Answer": gfr}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {"age": (49, 'years'), "creatinine": (10.6, 'mg/dL'), "sex": "Male"},
        {"age": (71, 'years'), "creatinine": (1.0, 'mg/dL'), "sex": "Female"},
    ]
    # {'age': [49, 'years'], 'creatinine': [10.6, 'mg/dL'], 'sex': 'Male'}
    # {'age': [71, 'years'], 'creatinine': [1.0, 'mg/dL'], 'sex': 'Female'}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = mrdr_gfr_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function framingham_risk_score_explanation
- translation

Date: March 2025
"""

import math

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.unit_converter_new import (
    conversion_explanation,
)


def framingham_risk_score_explanation(input_parameters):
    age_exp, age = age_conversion_explanation(input_parameters["age"])
    gender = input_parameters["sex"]

    explanation = f"The patient's gender is {gender}.\n"

    if gender == "Male":
        explanation += (
            "For males, the formula for computing the framingham "
            "risk score is 52.00961 * ln(age) + 20.014077 * ln("
            "total_cholestrol) + -0.905964 * ln(hdl_cholestrol) "
            "+ 1.305784 * ln(sys_bp) + 0.241549 * bp_medicine + "
            "12.096316 * smoker + -4.605038 * ln(age) * "
            "ln(total_cholestrol) + -2.84367 * ln(age_smoke) "
            "* smoker + -2.93323 * ln(age) * "
            "ln(age) -  172.300168, where 'bp_medicine' is a "
            "binary variable for whether a patient's blood "
            "pressure is being treated with medicine, total "
            "cholestrol and hdl cholestrol are in mg/dL, "
            "and 'smoker' is whether the patient is a "
            "smoker or not.\n"
        )
        explanation += (
            "From this, we use the risk score to get "
            "likelihood for a patient getting myocardial "
            "infraction (MI) or dying in the next 10 years: "
            "1 - 0.9402^exp(risk_score), where risk_score "
            "is the value from the formula above.\n"
        )
        explanation += age_exp

        if age > 70:
            age_smoke = 70
            explanation += (
                "For male patient's whose age is greater "
                "than 70, the age variable is set to 70 within "
                "the 'age' term for the β x ln(Age) x "
                "Smoker term.\n"
            )
        else:
            age_smoke = age

    if gender == "Female":
        explanation += (
            "For females, the formula for computing the "
            "framingham risk score is 31.764001 * ln(age) + "
            "22.465206 * ln(total_cholestrol) - 1.187731 * ln("
            "hdl_cholestrol) + 2.552905 * ln(sys_bp) + 0.420251 "
            "* bp_medicine + 13.07543 * smoker + -5.060998 "
            "* ln(age) * ln(total_cholestrol) + -2.996945 * "
            "ln(age_smoke) * smoker - 146.5933061, where "
            "'bp_medicine' is a binary variable for whether "
            "a patient's blood pressure is being treated "
            "with medicine, total cholestrol and hdl cholestrol "
            "are in mg/dL, and 'smoker' is whether the "
            "patient is a smoker or not.\n"
        )
        explanation += (
            "From this, we use the risk score to get likelihood "
            "for a patient getting myocardial infraction (MI) "
            "or dying in the next 10 years: 1 - "
            "0.9402^exp(risk_score), where risk_score "
            "is the value from the formula above.\n"
        )
        explanation += age_exp

        if age > 78:
            age_smoke = 78
            explanation += (
                "For female patient's whose age is greater than "
                "78, the age variable is set to 78 "
                "within the 'age' variable for the "
                "β x ln(Age) x Smoker term.\n"
            )
        else:
            age_smoke = age

    if "smoker" in input_parameters:
        if input_parameters["smoker"]:
            explanation += (
                "The patient is a smoker, making the "
                "smoking variable equal to 1.\n"
            )
            smoker = 1
        else:
            explanation += (
                "The patient is not a smoker, making "
                "the smoking variable equal to 0.\n"
            )
            smoker = 0
    else:
        explanation += (
            "The note does not specify whether the patient is a "
            "smoker and so we assume this to be false, "
            "making the smoking variable equal to 0.\n"
        )
        smoker = 0

    sys_bp = input_parameters["sys_bp"][0]

    explanation += (
        f"The patient's systolic blood pressure is {sys_bp} mm Hg.\n"
    )

    if "bp_medicine" in input_parameters:
        if input_parameters["bp_medicine"]:
            explanation += (
                "The patient has been specified to "
                "take medication for treating their "
                "blood pressure, making the bp_medicine "
                "variable equal to 1.\n"
            )
            bp_medicine = 1
        else:
            explanation += (
                "The patient has been specified to not "
                "take medication for treating their blood "
                "pressure, making the bp_medicine variable "
                "equal to 0.\n"
            )
            bp_medicine = 0
    else:
        explanation += (
            "The note does not specify whether the patient "
            "takes medicine for treating blood pressure and "
            "so we assume this to be false, making the "
            "bp_medicine variable equal to 0.\n"
        )
        bp_medicine = 0

    total_cholestrol_exp, total_cholestrol = conversion_explanation(
        input_parameters["total_cholestrol"][0],
        "total cholestrol",
        386.65,
        None,
        input_parameters["total_cholestrol"][1],
        "mmol/L",
    )
    hdl_cholestrol_exp, hdl_cholestrol = conversion_explanation(
        input_parameters["hdl_cholestrol"][0],
        "hdl cholestrol",
        386.65,
        None,
        input_parameters["hdl_cholestrol"][1],
        "mmol/L",
    )

    explanation += total_cholestrol_exp + '\n'
    explanation += hdl_cholestrol_exp + '\n'

    if gender == "Male":
        risk_score = round(
            52.00961 * math.log(age)
            + 20.014077 * math.log(total_cholestrol)
            - 0.905964 * math.log(hdl_cholestrol)
            + 1.305784 * math.log(sys_bp)
            + 0.241549 * bp_medicine
            + 12.096316 * smoker
            - 4.605038 * (math.log(age) * math.log(total_cholestrol))
            - 2.84367 * (math.log(age_smoke) * smoker)
            - 2.93323 * (math.log(age) * math.log(age))
            - 172.300168,
            1,
        )
        percentage = round(1 - 0.9402 ** math.exp(risk_score), 1)
        explanation += (
            f"Plugging in these values will give us "
            f"the risk score:  52.00961 * ln({age}) "
            f"+ 20.014077 * ln({total_cholestrol}) + -0.905964 * "
            f"ln({hdl_cholestrol}) + 1.305784 * "
            f"ln({sys_bp}) + 0.241549 * {bp_medicine}  "
            f"+ 12.096316 * {smoker} + -4.605038 * "
            f"ln({age}) * ln({total_cholestrol}) + "
            f"-2.84367 * ln({age_smoke}) * {smoker} "
            f"+ -2.93323 * ln({age}) * ln({age}) -  "
            f"172.300168 = {risk_score}.\n"
        )
        explanation += (
            f"Plugging this into the MI risk equation "
            f"gives us 1 - 0.9402^exp({risk_score}) = "
            f"{percentage}. We then multiply this by a "
            f"100 to obtain the percentage which "
            f"gives us {percentage} * 100 = "
            f"{round(percentage * 100, 3)}%.\n"
        )
    else:
        risk_score = round(
            31.764001 * math.log(age)
            + 22.465206 * math.log(total_cholestrol)
            - 1.187731 * math.log(hdl_cholestrol)
            + 2.552905 * math.log(sys_bp)
            + 0.420251 * bp_medicine
            + 13.07543 * smoker
            - 5.060998 * (math.log(age) * math.log(total_cholestrol))
            - 2.996945 * (math.log(age_smoke) * smoker)
            - 146.5933061,
            1,
        )
        percentage = round(1 - 0.98767 ** math.exp(risk_score), 1)
        explanation += (
            f"Plugging in these values will "
            f"give us the risk score: 31.764001 * "
            f"ln({age}) + 22.465206 * ln({total_cholestrol}) - "
            f"1.187731 * "
            f"ln({hdl_cholestrol}) + 2.552905 * "
            f"ln({sys_bp}) + 0.420251 * {bp_medicine} "
            f"+ 13.07543 * {smoker} + -5.060998 * "
            f"ln({age}) * ln({total_cholestrol}) + "
            f"-2.996945 * ln({age_smoke}) * {smoker} - "
            f"146.5933061 = {risk_score}.\n"
        )
        explanation += (
            f"Plugging this into the MI risk formula "
            f"gives us 1 - 0.98767^exp({risk_score}) = "
            f"{percentage}. We then multiply this by a "
            f"100 to obtain the percentage which "
            f"gives us {percentage} * 100 = "
            f"{round(percentage * 100, 3)}%.\n"
        )

    explanation += (
        f"The patient's percentage of getting MI or dying is"
        f" {round(percentage * 100, 3)} %.\n"
    )

    return {"Explanation": explanation, "Answer": round(percentage * 100, 3)}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'sex': 'Male',
            'age': (17, 'years'),
            'sys_bp': (70, "mm"),
            'dia_bp': (12, "Hg"),
            "smoker": True,
            "bp_medicine": True,
            "total_cholestrol": (210.0, 'mg/dL'),
            "hdl_cholestrol": (37.0, 'mg/dL'),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = framingham_risk_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
    'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- None

Date: March 2025
"""

from .rounding import round_number


def height_conversion_explanation(height_info):
    if len(height_info) == 4:
        inches = height_info[0] * 12 + height_info[2]
        meters = round_number(inches * 0.0254)

        return (
            f"The patient's height is {height_info[0]} ft {height_info[2]} "
            f"in which converts to {height_info[0]} ft * 12 in/ft + "
            f"{height_info[2]} in = {inches} in. Hence, the patient's "
            f"height is {inches} in * 0.0254 m/in = {meters} m. ",
            meters,
        )

    elif height_info[-1] == "m":
        return f"The patient's height is {height_info[0]} m. ", height_info[0]
    elif height_info[-1] == "cm":
        height = round_number(height_info[0] / 100)
        return (
            f"The patient's height is {height_info[0]} cm, which is "
            f"{height_info[0]} cm * 1 m / 100 cm = {height} m. ",
            height,
        )
    elif height_info[-1] == "ft":
        height = round_number(height_info[0] * 0.3048)
        return (
            f"The patient's height is {height_info[0]} ft, which is "
            f"{height_info[0]} ft * 0.3048 m / ft = {height} m. ",
            height,
        )
    elif height_info[-1] == "in":
        height = round_number(height_info[0] * 0.0254)
        return (
            f"The patient's height is {height_info[0]} in, which is "
            f"{height_info[0]} in * 0.0254 m / in = {height} m. ",
            height,
        )


def height_conversion_explanation_cm(height_info):
    if len(height_info) == 4:
        feet = height_info[0]
        inches = height_info[2]
        total_inches = feet * 12 + inches
        centimeters = round_number(total_inches * 2.54)
        explanation = (
            f"The patient's height is {feet} ft {inches} in which converts to "
            f"{feet} ft * 12 in/ft + {inches} in = {total_inches} in. "
            f"Hence, the patient's height is {total_inches} in * 2.54 cm/in "
            f"= {centimeters} cm. "
        )
        return explanation, centimeters

    elif height_info[-1] == "m":
        height_meters = height_info[0]
        centimeters = round_number(height_meters * 100)
        explanation = (
            f"The patient's height is {height_meters} m, which is "
            f"{height_meters} m * 100 cm/m = {centimeters} cm. "
        )
        return explanation, centimeters

    elif height_info[-1] == "cm":
        height_cm = height_info[0]
        explanation = f"The patient's height is {height_cm} cm. "
        return explanation, height_cm

    elif height_info[-1] == "ft":
        height_ft = height_info[0]
        centimeters = round_number(height_ft * 30.48)
        explanation = (
            f"The patient's height is {height_ft} ft, which is "
            f"{height_ft} ft * 30.48 cm/ft = {centimeters} cm. "
        )
        return explanation, centimeters

    elif height_info[-1] == "in":
        height_in = height_info[0]
        centimeters = round_number(height_in * 2.54)
        explanation = (
            f"The patient's height is {height_in} in, which is "
            f"{height_in} in * 2.54 cm/in = {centimeters} cm. "
        )
        return explanation, centimeters


def height_conversion_explanation_in(height_info):
    if len(height_info) == 4:
        feet = height_info[0]
        inches = height_info[2]
        total_inches = round_number(feet * 12 + inches)
        explanation = (
            f"The patient's height is {feet} ft {inches} in which converts to "
            f"{feet} ft * 12 in/ft + {inches} in = {total_inches} in. "
            f"Hence, the patient's height is {total_inches} in. "
        )
        return explanation, total_inches

    elif height_info[-1] == "m":
        height_meters = height_info[0]
        inches = round_number(height_meters * 39.3701)
        explanation = (
            f"The patient's height is {height_meters} m, which is "
            f"{height_meters} m * 39.3701 in/m = {inches} in. "
        )
        return explanation, inches

    elif height_info[-1] == "cm":
        height_cm = height_info[0]
        inches = round_number(height_cm * 0.393701)
        explanation = (
            f"The patient's height is {height_cm} cm, which is "
            f"{height_cm} cm * 0.393701 in/cm = {inches} in. "
        )
        return explanation, inches

    elif height_info[-1] == "ft":
        height_ft = height_info[0]
        inches = round_number(height_ft * 12)
        explanation = (
            f"The patient's height is {height_ft} ft, which is "
            f"{height_ft} ft * 12 in/ft = {inches} in. "
        )
        return explanation, inches

    elif height_info[-1] == "in":
        height_in = height_info[0]
        explanation = f"The patient's height is {height_in} in. "
        return explanation, height_in



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_albumin_corrected_delta_gap_explanation
- translation

Date: March 2025
"""

from .albumin_corrected_anion import (
    compute_albumin_corrected_anion_explanation,
)
from .utils.rounding import round_number


def compute_albumin_corrected_delta_gap_explanation(input_parameters):
    r"""
    Calculates the patient's albumin corrected delta gap and generates a
    detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "sodium" (array): The patient's blood sodium level in the
                format (value, unit).
                - Value (float): The blood sodium level.
                - Unit (str): The unit of blood sodium level.
            - "chloride" (array): The patient's chloride level in the
                format (value, unit).
                - Value (float): The value of chloride level.
                - Unit (str): The unit of chloride level, eg. "mmol/L",
                    "mEq/L", and so on.
            - "bicarbonate" (array): The patient's bicarbonate level in
                the format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                    "mEq/L", and so on.
            - "albumin" (array): The patient's albumin concentration in the
                format (value, unit).
                - Value (float): The numerical albumin concentration value.
                - Unit (str): The unit of albumin concentration, eg. "g/L",
                    "mg/dL", "g/mL" and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
                the calculation process.
            - "Answer" (float): The patient's albumin corrected delta gap.

    Notes:
        - None

    Example:
        compute_albumin_corrected_delta_gap_explanation()

        output: "{'Explanation': "To compute the formula of albumin
        corrected delta gap, the formula is albumin corrected anion gap (in
        mEq/L) - 12.\nThe formula for computing a patient's albumin
        corrected anion gap is:anion_gap (in mEq/L) + 2.5 * (4 -
        albumin (in g/dL)).\nThe formula for computing a patient's anion gap
        is:sodium (mEq/L) - (chloride (mEq/L)+ bicarbonate (mEq/L)).\nThe
        concentration of sodium is 141.0 mEq/L. \nThe concentration of
        chloride is 104.0 mEq/L. \nThe concentration of bicarbonate is 29.0
        mEq/L. \nPlugging in these values into the anion gap formula
        gives us 141.0 mEq/L - (104.0 mEq/L + 29.0 mEq/L) = 8.0 mEq/L.
        Hence, The patient's anion gap is 8.0 mEq/L.\nThe concentration of
        albumin is 43.0 g/L. We need to convert the concentration to g/dL.
        The mass units of the source and target are the same so no
        conversion is needed.The current volume unit is L and the target
        volume unit is dL. The conversion factor is 10.0 dL
        for every unit of L. Our next step will be to divide the mass by the
        volume conversion factor of 10.0 to get the final concentration in
        terms of g/dL. This will result to 43.0 g albumin/10.0 dL = 4.3 g
        albumin/dL. The concentration value of 43.0 g albumin/L converts to
        4.3 g albumin/dL. Plugging in these values into the albumin
        corrected anion gap formula, we get 8.0 (mEq/L) + 2.5 *
        (4 - 4.3 (in g/dL)) = 7.25 mEq/L. Hence,the patient's albumin
        corrected anion gap is 7.25 mEq/L.\nPlugging in 7.25 mEq/L for the
        anion gap into the albumin corrected delta gap formula, we get 7.25
        - 12 = -4.75 mEq/L. Hence, the patient's albumin corrected delta
        gap is -4.75 mEq/L.\n", 'Answer': -4.75}"
    """

    explanation = (
        "To compute the formula of albumin corrected delta gap, "
        "the formula is albumin corrected anion gap (in mEq/L) - 12.\n"
    )

    albumin_corrected_resp = compute_albumin_corrected_anion_explanation(
        input_parameters
    )

    explanation += albumin_corrected_resp["Explanation"]

    albumin_corrected_val = albumin_corrected_resp["Answer"]

    answer = round_number(albumin_corrected_val - 12.0)

    explanation += (
        f"Plugging in {albumin_corrected_val} mEq/L for the anion gap into "
        f"the albumin corrected delta gap formula, "
        f"we get {albumin_corrected_val} - 12 = {answer} mEq/L. "
    )
    explanation += (
        f"Hence, the patient's albumin corrected delta gap "
        f"is {answer} mEq/L.\n"
    )

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "chloride": (104.0, "mEq/L"),
            "bicarbonate": (29.0, "mEq/L"),
            "albumin": (43.0, "g/L"),
            "sodium": (141.0, "mEq/L"),
        },
        {
            "chloride": (89.0, "mmol/L"),
            "bicarbonate": (23.7, "mEq/L"),
            "albumin": (2.1, "g/dL"),
            "sodium": (133.0, "mmol/L"),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_albumin_corrected_delta_gap_explanation(
            input_variables
        )
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
    'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- None

Date: March 2025
"""

from math import floor, log10


def round_number(num):
    if num > 0.001:
        # Round to the nearest thousandth
        return round(num, 3)
    else:
        # Round to three significant digits
        if num == 0:
            return 0
        return round(num, -int(floor(log10(abs(num)))) + 2)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function psi_score_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.convert_temperature import (
    fahrenheit_to_celsius_explanation,
)
from .utils.unit_converter_new import (
    conversion_explanation,
)


def psi_score_explanation(input_variables):
    age_exp, age = age_conversion_explanation(input_variables["age"])
    gender = input_variables["sex"]
    pulse = input_variables["heart_rate"][0]
    temperature_exp, temperature = fahrenheit_to_celsius_explanation(
        input_variables["temperature"][0],
        input_variables["temperature"][1],
    )
    pH = input_variables["pH"]
    respiratory_rate = input_variables["respiratory_rate"][0]
    sys_bp = input_variables["sys_bp"][0]
    bun_exp, bun = conversion_explanation(
        input_variables["bun"][0],
        'BUN',
        28.02,
        None,
        input_variables["bun"][1],
        "mg/dL",
    )
    sodium_exp, sodium = conversion_explanation(
        input_variables["sodium"][0],
        "sodium",
        22.99,
        None,
        input_variables["sodium"][1],
        "mmol/L",
    )
    glucose_exp, glucose = conversion_explanation(
        input_variables["glucose"][0],
        "glucose",
        180.16,
        None,
        input_variables["glucose"][1],
        "mg/dL",
    )
    hemocratit = input_variables["hemocratit"][0]
    partial_pressure_oxygen = input_variables.get("partial_pressure_oxygen")

    explanation = """
    The rules for computing the Pneumonia Severity Index (PSI) are shown below:
    
       1. Age: Enter age in years (age score will be equal to age in years)
       2. Sex: Female = -10 points, Male = 0 points
       3. Nursing home resident: No = 0 points, Yes = +10 points
       4. Neoplastic disease: No = 0 points, Yes = +30 points
       5. Liver disease history: No = 0 points, Yes = +20 points
       6. Congestive heart failure (CHF) history: No = 0 points, Yes = +10 
       points
       7. Cerebrovascular disease history: No = 0 points, Yes = +10 points
       8. Renal disease history: No = 0 points, Yes = +10 points
       9. Altered mental status: No = 0 points, Yes = +20 points
       10. Respiratory rate ≥30 breaths/min: No = 0 points, Yes = +20 points
       11. Systolic blood pressure <90 mmHg: No = 0 points, Yes = +20 points
       12. Temperature <35°C (95°F) or >39.9°C (103.8°F): No = 0 points, 
       Yes = +15 points
       13. Pulse ≥125 beats/min: No = 0 points, Yes = +10 points
       14. pH <7.35: No = 0 points, Yes = +30 points
       15. BUN ≥30 mg/dL or ≥11 mmol/L: No = 0 points, Yes = +20 points
       16. Sodium <130 mmol/L: No = 0 points, Yes = +20 points
       17. Glucose ≥250 mg/dL or ≥14 mmol/L: No = 0 points, Yes = +10 points
       18. Hematocrit <30%: No = 0 points, Yes = +10 points
       19. Partial pressure of oxygen <60 mmHg or <8 kPa: No = 0 points, 
       Yes = +10 points
       20. Pleural effusion on x-ray: No = 0 points, Yes = +10 points
    
    The total score is calculated by summing the points for each criterion.\n\n
    """

    explanation += "The current PSI score is 0.\n"
    age_explanation, age = age_conversion_explanation(input_variables["age"])
    explanation += age_explanation
    explanation += (
        f"We add the the number of years of age of the "
        f"patient to the psi score, making the current "
        f"total 0 + {age} = {age}.\n"
    )

    psi_score = 0

    psi_score += age

    if gender == "Female":
        explanation += (
            f"Because the patient is female, we subtract "
            f"10 points from the current total, making the "
            f"current total {psi_score} - 10 "
            f"= {psi_score - 10}.\n"
        )
        psi_score -= 10
    else:
        explanation += (
            f"Because the patient is male, no adjustments "
            f"are made to the score, keeping the "
            f"current total at {psi_score}.\n"
        )

    parameters = {
        "nursing_home_resident": ("Nursing Home Resident", 10),
        "neoplastic_disease": ("Neoplastic disease", 30),
        "liver_disease": ("Liver disease history", 20),
        "chf": ("CHF History", 10),
        "cerebrovascular_disease": ("Cerebrovascular disease history", 10),
        "renal_disease": ("Renal Disease History", 10),
        "altered_mental_status": ("Altered Mental Status", 20),
        "pleural_effusion": ("Pleural effusion on x-ray", 10),
    }

    for parameter in parameters:
        if parameter == 'nursing_home_resident':
            if parameter not in input_variables:
                explanation += (
                    f"Whether patient is a nursing home "
                    f"resident is not reported. Hence, "
                    f"we assume this to be false and so "
                    f"not add any points to the current "
                    f"total keeping it at {psi_score}.\n"
                )
            elif not input_variables[parameter]:
                explanation += (
                    f"The patient is not a nursing home "
                    f"resident and so we do not add any "
                    f"points to the current total "
                    f"keeping it at {psi_score}.\n"
                )
            else:
                explanation += (
                    f"The patient is reported to be a "
                    f"nursing home resident and so we "
                    f"add 10 points to the score, "
                    f"making the current total "
                    f"{psi_score} + 10 = {psi_score + 10}.\n"
                )
                psi_score += 10
            continue

        if parameter not in input_variables:
            explanation += (
                f"{parameters[parameter][0]} is not reported "
                f"for the patient and so we assume it to be "
                f"false. Hence, we do not add any points "
                f"to the current total keeping it at {psi_score}.\n"
            )
        elif not input_variables[parameter]:
            explanation += (
                f"{parameters[parameter][0]} is reported "
                f"to be false for the patient and so we do "
                f"not add any points to the current total "
                f"keeping it at {psi_score}.\n"
            )
        elif input_variables[parameter]:
            points = parameters[parameter][1]
            explanation += (
                f"{parameters[parameter][0]} is reported to "
                f"be present for the patient and so we add "
                f"{points} points to the score, making "
                f"the current total {psi_score} + {points} "
                f"= {psi_score + points}.\n"
            )
            psi_score += points

    explanation += f"The patient's pulse is {pulse} beats per minute. "

    if pulse >= 125:
        explanation += (
            f"The pulse is greater or equal to than 125 beats "
            f"per minute, and so we add 10 points to the "
            f"score, making the current total {psi_score} "
            f"+ 10 = {psi_score + 10}.\n"
        )
        psi_score += 10
    else:
        explanation += (
            f"The pulse is less than 125 beats per minute "
            f"and so we do not add any points to the score, "
            f"keeping it at {psi_score}.\n"
        )

    explanation += temperature_exp
    if temperature < 35:
        explanation += (
            f"The patient's temperature is less than 35 "
            f"degrees celsius, and so we add 15 points to "
            f"the score, making the current total {psi_score} "
            f"+ 15 = {psi_score + 15}.\n"
        )
        psi_score += 15
    elif temperature > 39.9:
        explanation += (
            f"The patient's temperature is greater than "
            f"39.9 degrees celsius and so we add 15 points "
            f"to the score, making the current total "
            f"{psi_score} + 15 = {psi_score + 15}.\n"
        )
        psi_score += 15
    else:
        explanation += (
            f"The patient's temperature is greater than 35 "
            f"degrees celsius and the temperature is less "
            f"than 39.9 degrees celsius, and so we do not "
            f"add any points to the score, keeping the "
            f"total at {psi_score}.\n"
        )

    explanation += f"The patient's pH is {pH}. "

    if pH < 7.35:
        explanation += (
            f"The patient's pH is less than 7.35, and so "
            f"we add 30 points to the score, making the "
            f"current total {psi_score} + 30 = {psi_score + 30}.\n"
        )
        psi_score += 30
    else:
        explanation += (
            f"The patient's pH is greater than or equal "
            f"to 7.35, and so we do not add any points "
            f"to the score, keeping the current total "
            f"at {psi_score}.\n"
        )

    explanation += (
        f"The patient's respiratory rate is {respiratory_rate} "
        f"breaths per minute. "
    )

    if respiratory_rate >= 30:
        explanation += (
            f"The patient's respiratory rate is greater "
            f"than or equal to 30 breaths per minute and "
            f"so we add 20 points to the score, making "
            f"current total {psi_score} + 20 = {psi_score + 20}.\n"
        )
        psi_score += 20
    else:
        explanation += (
            f"The patient's respiratory rate is less than "
            f"30 breaths per minute and so we do not add "
            f"any points to the score, keeping the total "
            f"score at {psi_score}.\n"
        )

    explanation += f"The patient's systolic blood pressure is {sys_bp} mm Hg. "

    if sys_bp < 90:
        explanation += (
            f"The patient's systolic blood pressure is "
            f"less than 90 mm Hg and so we add 20 points "
            f"to the score, making current total "
            f"{psi_score} + 20 = {psi_score + 20}.\n"
        )
        psi_score += 20
    else:
        explanation += (
            f"The patient's systolic blood pressure is "
            f"greater than or equal to 90 mm Hg and so "
            f"we do not add any points to the score, "
            f"keeping the total at {psi_score}.\n"
        )

    explanation += bun_exp

    if bun >= 30:
        explanation += (
            f"The patient's BUN is greater than or equal "
            f"to 30 mg/dL, and so we add 20 points to "
            f"the score, making current total {psi_score} "
            f"+ 20 = {psi_score + 20}.\n"
        )
        psi_score += 20
    else:
        explanation += (
            f"The patient's BUN is less than 30 mg/dL, "
            f"and so we do not add any points to the "
            f"score, keeping the total at {psi_score}.\n"
        )

    explanation += sodium_exp

    if sodium < 130:
        explanation += (
            f"The patient's sodium is less than 130 "
            f"mmol/L, and so we add 20 points to the "
            f"score, making the current total "
            f"{psi_score} + 20 = {psi_score + 20}.\n"
        )
        psi_score += 20
    else:
        explanation += (
            f"The patient's sodium is greater than or "
            f"equal to 130 mmol/L, and so we do not "
            f"add any points to the score, keeping "
            f"the total at {psi_score}.\n"
        )

    explanation += glucose_exp

    if glucose >= 250:
        explanation += (
            f"The patient's glucose concentration is "
            f"greater than 250 mg/dL, and so we add 10 "
            f"points to the score, making the current "
            f"total {psi_score} + 10 = {psi_score + 10}.\n"
        )
        psi_score += 10
    else:
        explanation += (
            f"The patient's glucose concentration is "
            f"less than or equal to than 250 mg/dL, "
            f"and so we not add any points to the current "
            f"total, keeping it at {psi_score}.\n"
        )

    explanation += f"The patient's hemocratit is {hemocratit} %. "

    if hemocratit < 30:
        explanation += (
            f"The patient's hemocratit is less than 30%, "
            f"and so we add 10 points to the score, "
            f"making the current total {psi_score} + "
            f"10 = {psi_score + 10}.\n"
        )
        psi_score += 10
    else:
        explanation += (
            f"The patient's hemocratit is greater than or equal "
            f"to 30%, and so we not add any points to "
            f"the current total, keeping it at {psi_score}.\n"
        )

    if partial_pressure_oxygen[1] == "mm Hg":
        explanation += (
            f"The patient's partial pressure of oxygen "
            f"is {partial_pressure_oxygen[0]} mm Hg. "
        )

        if partial_pressure_oxygen[0] < 60:
            explanation += (
                f"The patient's partial pressure of "
                f"oxygen is less than 60 mm Hg, and so "
                f"we add {psi_score} points to the score, "
                f"making the current total {psi_score} "
                f"+ 10 = {psi_score + 10}.\n"
            )
            psi_score += 10
        else:
            explanation += (
                f"The patient's partial pressure of "
                f"oxygen is greater than or equal to "
                f"60 mm Hg, and so we not add any points "
                f"to the current total, keeping "
                f"it at {psi_score}.\n"
            )

    elif partial_pressure_oxygen[1] == "kPa":
        explanation += (
            f"The patient's partial pressure of oxygen "
            f"is {partial_pressure_oxygen[0]} kPa. "
        )

        if partial_pressure_oxygen[0] < 8:
            explanation += (
                f"The patient's partial pressure of "
                f"oxygen is less than 8 kPa, and so we "
                f"add {psi_score} points to the score, "
                f"making the current total {psi_score} "
                f"+ 10 = {psi_score + 10}.\n"
            )
            psi_score += 10
        else:
            explanation += (
                f"The patient's partial pressure of "
                f"oxygen is greater than or equal to 8 "
                f"kPa, and so we not add any points to "
                f"the current total, keeping it at {psi_score}.\n"
            )

    explanation += f"The patient's PSI score is {psi_score}.\n"

    return {"Explanation": explanation, "Answer": psi_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "sex": "Male",
            "age": (17, "years"),
            "temperature": (99.0, "degrees fahreinheit"),
            "heart_rate": (98.0, 'beats per minute'),
            "pH": 7.3,
            "respiratory_rate": (17.0, "breaths per minute"),
            "sys_bp": (70, "mm"),
            "bun": (3.5, "mmol/L"),
            "sodium": (134.0, "mmol/L"),
            "glucose": (97.3, "mg/dL"),
            "hemocratit": (20, "%"),
            "partial_pressure_oxygen": (80, "mm"),
            "nursing_home_resident": False,  # Optional
            "neoplastic_disease": False,  # Optional
            "liver_disease": False,  # Optional
            "chf": False,  # Optional
            "cerebrovascular_disease": False,  # Optional
            "renal_disease": False,  # Optional
            "altered_mental_status": False,  # Optional
            "pleural_effusion": False,  # Optional
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = psi_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function sirs_criteria_explanation
- translation

Date: March 2025
"""

from .utils.convert_temperature import (
    fahrenheit_to_celsius_explanation,
)
from .utils.unit_converter_new import (
    convert_to_units_per_liter_explanation,
)


def sirs_criteria_explanation(input_parameters):
    explanation = r"""
    The rules for SIRS Criteria are listed below:

       1. Temperature >38°C (100.4°F) or <36°C (96.8°F): No = 0 points, 
       Yes = +1 point
       2. Heart rate >90: No = 0 points, Yes = +1 point
       3. Respiratory rate >20 or PaCO₂ <32 mm Hg: No = 0 points, Yes = +1 
       point
       4. White blood cell count (WBC) >12,000/mm³, <4,000/mm³, or >10% 
       bands: No = 0 points, Yes = +1 point

    The total number of criteria met is taken by summing the score for each 
    criteria.\n\n
    """

    explanation += "The current count of SIRS criteria met is 0.\n"

    temperature = input_parameters["temperature"]

    temp_exp, temperature = fahrenheit_to_celsius_explanation(
        temperature[0], temperature[1]
    )
    heart_rate = input_parameters["heart_rate"][0]
    wbc_exp, wbc = convert_to_units_per_liter_explanation(
        input_parameters["wbc"][0],
        input_parameters["wbc"][1],
        "white blood cell",
        "m^3",
    )

    criteria_met = 0

    explanation += temp_exp

    if temperature > 38:
        explanation += (
            f"Because the temperature is greater than 38 degrees "
            f"celsius, we increment the criteria count by 1 "
            f"making the current total {criteria_met} + 1 = "
            f"{criteria_met + 1}.\n"
        )
        criteria_met += 1
    elif temperature < 36:
        explanation += (
            f"Because the temperature is less than 36 degrees "
            f"celsius, we increment the criteria count by 1 "
            f"making the current total {criteria_met} + 1 = "
            f"{criteria_met + 1}.\n"
        )
        criteria_met += 1
    else:
        explanation += (
            f"Because the temperature is between 36 and 38 "
            f"degrees celsius, this does not meet SIRS criteria "
            f"for temperature, and so the current total remains "
            f"at {criteria_met}.\n"
        )

    explanation += (
        f"The patient's heart rate is {heart_rate} beats per " f"minute. "
    )

    if heart_rate > 90:
        explanation += (
            f"Because the heart rate is greater than 90 beats "
            f"per minute, this meets SIRS criteria and so we "
            f"increment the criteria count by 1 making the "
            f"current total {criteria_met} + 1 = "
            f"{criteria_met + 1}.\n"
        )
        criteria_met += 1
    else:
        explanation += (
            f"Because the heart rate is less than 90 beats per "
            f"minute, this does not meet SIRS criteria for heart "
            f"rate, and so the current total remains at "
            f"{criteria_met}.\n"
        )
        criteria_met += 1

    explanation += wbc_exp

    if wbc > 12000:
        explanation += (
            f"Because the white blood cell count is greater than "
            f"12000 count per mm^3, we increment the criteria "
            f"count by 1 making the current total {criteria_met} "
            f"+ 1 = {criteria_met + 1}.\n"
        )
        criteria_met += 1
    elif wbc < 4000:
        explanation += (
            f"Because the white blood cell count is less than "
            f"4000 count per mm^3, we increment the criteria "
            f"count by 1 making the current total {criteria_met} "
            f"+ 1 = {criteria_met + 1}.\n"
        )
        criteria_met += 1
    else:
        explanation += (
            f"Because the white blood cell count is between 4000 "
            f"and 12000 count per mm^3, this does not meet SIRS "
            f"criteria for white blood cell count, and so the "
            f"current total remains at {criteria_met}.\n"
        )

    explanation += (
        "The final SIRS criteria is whether the patient has a "
        "respiratory rate of more than 20 breaths per minute or "
        "if the patient's PaCO₂ partial pressure is less than 32 "
        "mm Hg. "
    )

    if 'respiratory_rate' in input_parameters:
        respiratory_rate = input_parameters['respiratory_rate'][0]
        explanation += (
            f"The patient's respiratory rate is "
            f"{respiratory_rate} breaths per minute, "
        )
        res = ""

        if respiratory_rate > 20:
            res += "which is greater than 20 breaths per minute. "
            resp_met = True
        else:
            res += "which is less or equal to than 20 breaths per min. "
            resp_met = False

        explanation += res
    else:
        explanation += (
            "The patient's respiratory rate is not provided and "
            "so we assume that the patient's respiratory rate is "
            "less than or equal to 20 breaths per minute. "
        )
        resp_met = False

    if 'paco2' in input_parameters:
        paco2 = input_parameters['paco2'][0]
        paco2_unit = input_parameters['paco2'][1]
        if paco2 is not None and paco2_unit is not None:
            explanation += (
                f"The patient's PaCO₂ partial pressure"
                f"is {paco2} {paco2_unit}, "
            )
            if paco2 < 32:
                explanation += "which is less than 32 mm Hg. "
                paco2_met = True
            else:
                explanation += "which is greater than or equal to 32 mm Hg. "
                paco2_met = False
        else:
            explanation += (
                "The patient's PaCO₂ partial pressure data is not available. "
                "We cannot assess this part of the criteria. "
            )
            paco2_met = False
    else:
        explanation += (
            "The patient's PaCO₂ partial pressure is not provided. "
            "We cannot assess this part of the criteria."
        )
        paco2_met = False

    if resp_met or paco2_met:
        explanation += (
            f"At least one of the criteria is met, and so we "
            f"increment the criteria count by 1 giving us "
            f"a total of {criteria_met} + 1 = "
            f"{criteria_met + 1} criteria met.\n"
        )
        criteria_met += 1
    else:
        explanation += (
            f"Neither criteria met and so keep the current total "
            f"at {criteria_met}.\n"
        )

    explanation += (
        f"Hence, the the number of SIRS criteria met by the "
        f"patient is {criteria_met}.\n"
    )

    return {"Explanation": explanation, "Answer": criteria_met}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "temperature": (38.0, 'degrees celsius'),
            "heart_rate": (74.0, 'beats per minute'),
            "respiratory_rate": (50.0, 'breaths per minute'),
            "wbc": (15760.0, 'mL'),
        },
        {
            'temperature': (38.9, 'degrees celsius'),
            'heart_rate': (98.0, 'beats per minute'),
            'respiratory_rate': (17.0, 'breaths per minute'),
            'wbc': (10500.0, 'm^3'),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = sirs_criteria_explanation(input_variables)
        print(result)
        print("-" * 50)
